<?php 
/**
 * Plugin Name: Exclude Countries (Matomo Plugin)
 * Plugin URI: http://plugins.matomo.org/ExcludeCountries
 * Description: Exclude or include visitors from selected countries
 * Author: Lukas Winkler
 * Author URI: https://lw1.at
 * Version: 0.3.0
 */
?><?php
/**
 * Matomo - free/libre analytics platform
 *
 * @link https://matomo.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html GPL v3 or later
 */

namespace Piwik\Plugins\ExcludeCountries;

use Piwik\Common;
use Piwik\Container\StaticContainer;
use Piwik\Piwik;
use Piwik\Plugin;
use Piwik\Plugins\UserCountry\LocationProvider;
use Piwik\Settings\Setting;
use Piwik\Tracker\Request;

 
if (defined( 'ABSPATH')
&& function_exists('add_action')) {
    $path = '/matomo/app/core/Plugin.php';
    if (defined('WP_PLUGIN_DIR') && WP_PLUGIN_DIR && file_exists(WP_PLUGIN_DIR . $path)) {
        require_once WP_PLUGIN_DIR . $path;
    } elseif (defined('WPMU_PLUGIN_DIR') && WPMU_PLUGIN_DIR && file_exists(WPMU_PLUGIN_DIR . $path)) {
        require_once WPMU_PLUGIN_DIR . $path;
    } else {
        return;
    }
    add_action('plugins_loaded', function () {
        if (function_exists('matomo_add_plugin')) {
            matomo_add_plugin(__DIR__, __FILE__, true);
        }
    });
}

class ExcludeCountries extends Plugin
{
    /**
     * inspired by plugins/UserCountry/Columns/Country.php
     */
    public static function listCountries() {
        $regionDataProvider = StaticContainer::get('Piwik\Intl\Data\Provider\RegionDataProvider');
        $countryList = $regionDataProvider->getCountryList();
        array_walk($countryList, function (&$item, $key) {
            $item = Piwik::translate('Intl_Country_' . strtoupper($key));
        });
        asort($countryList); //order by localized name
        return $countryList;
    }

    public function registerEvents() {
        return [
            "Tracker.isExcludedVisit" => "checkExcludedCountry",
        ];
    }

    public function checkExcludedCountry(&$excluded, Request $request) {
        $logger = StaticContainer::getContainer()->get("Psr\Log\LoggerInterface");
        $provider = LocationProvider::getProviderById(Common::getCurrentLocationProviderId());
        $settings = new SystemSettings();


        $location = $provider->getLocation(["ip" => $request->getIpString()]);
        $countryCode = strtolower($location[LocationProvider::COUNTRY_CODE_KEY]);
        if (!$location || empty($countryCode)) {
            $logger->debug("ExcludeCountries could not detect a location");
            return;
        }
        $excludeBool = $settings->excludeBool->getValue();

        if ($excludeBool) {
            $countries = $this->settingToCountryCodes($settings->excludedCountries);
            if (in_array($countryCode, $countries)) {
                $logger->debug("request excluded by ExcludeCountries plugin (" . $countryCode . ")");
                $excluded = true;
            }
        } else {
            $countries = $this->settingToCountryCodes($settings->includedCountries);
            if (!in_array($countryCode, $countries)) {
                $logger->debug("request excluded by ExcludeCountries plugin (" . $countryCode . ")");
                $excluded = true;
            }
        }
    }

    private function settingToCountryCodes(Setting $setting) {
        $codes = [];
        foreach ($setting->getValue() as $value) {
            if (!empty($value["country"])) {
                $codes[] = $value["country"];
            }
        }
        return $codes;
    }

}
